@@ -38,11 +38,10 @@
  *
  * This code is released under the terms of the PostgreSQL License.
  *
- * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
- * Portions taken from FreeBSD.
  *
- * $PostgreSQL: pgsql/src/bin/initdb/initdb.c,v 1.186 2010/02/26 02:01:15 momjian Exp $
+ * src/bin/initdb/initdb.c
  *
  *-------------------------------------------------------------------------
  */
@@ -83,6 +82,7 @@ static char *username = "";
 static bool pwprompt = false;
 static char *pwfilename = NULL;
 static char *authmethod = "";
+static char *authmethodlocal = "";
 static bool debug = false;
 static bool noclean = false;
 static bool show_setting = false;
@@ -152,11 +152,9 @@ static char **filter_lines_with_token(char **lines, const char *token);
 static char **readfile(const char *path);
 static void writefile(char *path, char **lines);
 static FILE *popen_check(const char *command, const char *mode);
-static int	mkdir_p(char *path, mode_t omode);
 static void exit_nicely(void);
 static char *get_id(void);
 static char *get_encoding_id(char *encoding_name);
-static int	check_data_dir(char *dir);
 static bool mkdatadir(const char *subdir);
 static void set_input(char **dest, char *filename);
 static void check_input(char *path);
@@ -170,6 +168,7 @@ static void get_set_pwd(void);
 static void setup_depend(void);
 static void setup_sysviews(void);
 static void setup_description(void);
+static void setup_collation(void);
 static void setup_conversion(void);
 static void setup_dictionary(void);
 static void setup_privileges(void);
@@ -229,6 +228,12 @@ do { \
 		output_failed = true, output_errno = errno; \
 } while (0)
 
+#define PG_CMD_PRINTF3(fmt, arg1, arg2, arg3)		\
+do { \
+	if (fprintf(cmdfd, fmt, arg1, arg2, arg3) < 0 || fflush(cmdfd) < 0) \
+		output_failed = true, output_errno = errno; \
+} while (0)
+
 #ifndef WIN32
 #define QUOTE_PATH	""
 #define DIR_SEP "/"
@@ -471,110 +476,6 @@ popen_check(const char *command, const char *mode)
 	return cmdfd;
 }
 
-/* source stolen from FreeBSD /src/bin/mkdir/mkdir.c and adapted */
-
-/*
- * this tries to build all the elements of a path to a directory a la mkdir -p
- * we assume the path is in canonical form, i.e. uses / as the separator
- * we also assume it isn't null.
- *
- * note that on failure, the path arg has been modified to show the particular
- * directory level we had problems with.
- */
-static int
-mkdir_p(char *path, mode_t omode)
-{
-	struct stat sb;
-	mode_t		numask,
-				oumask;
-	int			first,
-				last,
-				retval;
-	char	   *p;
-
-	p = path;
-	oumask = 0;
-	retval = 0;
-
-#ifdef WIN32
-	/* skip network and drive specifiers for win32 */
-	if (strlen(p) >= 2)
-	{
-		if (p[0] == '/' && p[1] == '/')
-		{
-			/* network drive */
-			p = strstr(p + 2, "/");
-			if (p == NULL)
-				return 1;
-		}
-		else if (p[1] == ':' &&
-				 ((p[0] >= 'a' && p[0] <= 'z') ||
-				  (p[0] >= 'A' && p[0] <= 'Z')))
-		{
-			/* local drive */
-			p += 2;
-		}
-	}
-#endif
-
-	if (p[0] == '/')			/* Skip leading '/'. */
-		++p;
-	for (first = 1, last = 0; !last; ++p)
-	{
-		if (p[0] == '\0')
-			last = 1;
-		else if (p[0] != '/')
-			continue;
-		*p = '\0';
-		if (!last && p[1] == '\0')
-			last = 1;
-		if (first)
-		{
-			/*
-			 * POSIX 1003.2: For each dir operand that does not name an
-			 * existing directory, effects equivalent to those caused by the
-			 * following command shall occcur:
-			 *
-			 * mkdir -p -m $(umask -S),u+wx $(dirname dir) && mkdir [-m mode]
-			 * dir
-			 *
-			 * We change the user's umask and then restore it, instead of
-			 * doing chmod's.
-			 */
-			oumask = umask(0);
-			numask = oumask & ~(S_IWUSR | S_IXUSR);
-			(void) umask(numask);
-			first = 0;
-		}
-		if (last)
-			(void) umask(oumask);
-
-		/* check for pre-existing directory; ok if it's a parent */
-		if (stat(path, &sb) == 0)
-		{
-			if (!S_ISDIR(sb.st_mode))
-			{
-				if (last)
-					errno = EEXIST;
-				else
-					errno = ENOTDIR;
-				retval = 1;
-				break;
-			}
-		}
-		else if (mkdir(path, last ? omode : S_IRWXU | S_IRWXG | S_IRWXO) < 0)
-		{
-			retval = 1;
-			break;
-		}
-		if (!last)
-			*p = '/';
-	}
-	if (!first && !last)
-		(void) umask(oumask);
-	return retval;
-}
-
 /*
  * clean up any files we created on failure
  * if we created the data directory remove it too
@@ -802,53 +703,6 @@ find_matching_ts_config(const char *lc_type)
 }
 
 
-/*
- * make sure the directory either doesn't exist or is empty
- *
- * Returns 0 if nonexistent, 1 if exists and empty, 2 if not empty,
- * or -1 if trouble accessing directory
- */
-static int
-check_data_dir(char *dir)
-{
-	DIR		   *chkdir;
-	struct dirent *file;
-	int			result = 1;
-
-	errno = 0;
-
-	chkdir = opendir(dir);
-
-	if (!chkdir)
-		return (errno == ENOENT) ? 0 : -1;
-
-	while (errno = 0, (file = readdir(chkdir)) != NULL)
-	{
-		if (strcmp(".", file->d_name) == 0 ||
-			strcmp("..", file->d_name) == 0)
-		{
-			/* skip this and parent directory */
-			continue;
-		}
-		else
-		{
-			result = 2;			/* not empty */
-			break;
-		}
-	}
-
-#ifdef WIN32
-	/* Bug in old Mingw dirent.c;  fixed in mingw-runtime-3.2, 2003-10-10 */
-	if (GetLastError() == ERROR_NO_MORE_FILES)
-		errno = 0;
-#endif
-
-	if (errno || closedir(chkdir))
-		result = -1;			/* some kind of I/O error? */
-
-	return result;
-}
-
 /*
  * make the data directory (or one of its subdirectories if subdir is not NULL)
  */
@@ -865,7 +719,7 @@ mkdatadir(const char *subdir)
 	else
 		strcpy(path, pg_data);
 
-	if (mkdir_p(path, 0700) == 0)
+	if (pg_mkdir_p(path, S_IRWXU) == 0)
 		return true;
 
 	fprintf(stderr, _("%s: could not create directory \"%s\": %s\n"),
@@ -1161,7 +1015,7 @@ setup_config(void)
 	snprintf(path, sizeof(path), "%s/postgresql.conf", pg_data);
 
 	writefile(path, conflines);
-	chmod(path, 0600);
+	chmod(path, S_IRUSR | S_IWUSR);
 
 	free(conflines);
 
@@ -1224,15 +1078,23 @@ setup_config(void)
 	conflines = replace_token(conflines,
 							  "@authmethod@",
 							  authmethod);
+	conflines = replace_token(conflines,
+							  "@authmethodlocal@",
+							  authmethodlocal);
 
 	conflines = replace_token(conflines,
 							  "@authcomment@",
 					   strcmp(authmethod, "trust") ? "" : AUTHTRUST_WARNING);
 
+	/* Replace username for replication */
+	conflines = replace_token(conflines,
+							  "@default_username@",
+							  username);
+
 	snprintf(path, sizeof(path), "%s/pg_hba.conf", pg_data);
 
 	writefile(path, conflines);
-	chmod(path, 0600);
+	chmod(path, S_IRUSR | S_IWUSR);
 
 	free(conflines);
 
@@ -1243,7 +1105,7 @@ setup_config(void)
 	snprintf(path, sizeof(path), "%s/pg_ident.conf", pg_data);
 
 	writefile(path, conflines);
-	chmod(path, 0600);
+	chmod(path, S_IRUSR | S_IWUSR);
 
 	free(conflines);
 
@@ -1536,6 +1398,8 @@ setup_depend(void)
 		" FROM pg_ts_template;\n",
 		"INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' "
 		" FROM pg_ts_config;\n",
+		"INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' "
+		" FROM pg_collation;\n",
 		"INSERT INTO pg_shdepend SELECT 0,0,0,0, tableoid,oid, 'p' "
 		" FROM pg_authid;\n",
 		NULL
@@ -1642,11 +1506,205 @@ setup_description(void)
 				"  FROM tmp_pg_shdescription t, pg_class c "
 				"   WHERE c.relname = t.classname;\n");
 
+	/* Create default descriptions for operator implementation functions */
+	PG_CMD_PUTS("WITH funcdescs AS ( "
+				"SELECT p.oid as p_oid, oprname, "
+			  "coalesce(obj_description(o.oid, 'pg_operator'),'') as opdesc "
+				"FROM pg_proc p JOIN pg_operator o ON oprcode = p.oid ) "
+				"INSERT INTO pg_description "
+				"  SELECT p_oid, 'pg_proc'::regclass, 0, "
+				"    'implementation of ' || oprname || ' operator' "
+				"  FROM funcdescs "
+				"  WHERE opdesc NOT LIKE 'deprecated%' AND "
+				"  NOT EXISTS (SELECT 1 FROM pg_description "
+		  "    WHERE objoid = p_oid AND classoid = 'pg_proc'::regclass);\n");
+
 	PG_CMD_CLOSE;
 
 	check_ok();
 }
 
+#ifdef HAVE_LOCALE_T
+/*
+ * "Normalize" a locale name, stripping off encoding tags such as
+ * ".utf8" (e.g., "en_US.utf8" -> "en_US", but "br_FR.iso885915@euro"
+ * -> "br_FR@euro").  Return true if a new, different name was
+ * generated.
+ */
+static bool
+normalize_locale_name(char *new, const char *old)
+{
+	char	   *n = new;
+	const char *o = old;
+	bool		changed = false;
+
+	while (*o)
+	{
+		if (*o == '.')
+		{
+			/* skip over encoding tag such as ".utf8" or ".UTF-8" */
+			o++;
+			while ((*o >= 'A' && *o <= 'Z')
+				   || (*o >= 'a' && *o <= 'z')
+				   || (*o >= '0' && *o <= '9')
+				   || (*o == '-'))
+				o++;
+			changed = true;
+		}
+		else
+			*n++ = *o++;
+	}
+	*n = '\0';
+
+	return changed;
+}
+#endif   /* HAVE_LOCALE_T */
+
+/*
+ * populate pg_collation
+ */
+static void
+setup_collation(void)
+{
+#if defined(HAVE_LOCALE_T) && !defined(WIN32)
+	int			i;
+	FILE	   *locale_a_handle;
+	char		localebuf[NAMEDATALEN];
+	int			count = 0;
+
+	PG_CMD_DECL;
+#endif
+
+	fputs(_("creating collations ... "), stdout);
+	fflush(stdout);
+
+#if defined(HAVE_LOCALE_T) && !defined(WIN32)
+	snprintf(cmd, sizeof(cmd),
+			 "\"%s\" %s template1 >%s",
+			 backend_exec, backend_options,
+			 DEVNULL);
+
+	locale_a_handle = popen_check("locale -a", "r");
+	if (!locale_a_handle)
+		return;					/* complaint already printed */
+
+	PG_CMD_OPEN;
+
+	PG_CMD_PUTS("CREATE TEMP TABLE tmp_pg_collation ( "
+				"	collname name, "
+				"	locale name, "
+				"	encoding int) WITHOUT OIDS;\n");
+
+	while (fgets(localebuf, sizeof(localebuf), locale_a_handle))
+	{
+		size_t		len;
+		int			enc;
+		bool		skip;
+		char	   *quoted_locale;
+		char		alias[NAMEDATALEN];
+
+		len = strlen(localebuf);
+
+		if (len == 0 || localebuf[len - 1] != '\n')
+		{
+			if (debug)
+				fprintf(stderr, _("%s: locale name too long, skipped: %s\n"),
+						progname, localebuf);
+			continue;
+		}
+		localebuf[len - 1] = '\0';
+
+		/*
+		 * Some systems have locale names that don't consist entirely of ASCII
+		 * letters (such as "bokm&aring;l" or "fran&ccedil;ais").  This is
+		 * pretty silly, since we need the locale itself to interpret the
+		 * non-ASCII characters. We can't do much with those, so we filter
+		 * them out.
+		 */
+		skip = false;
+		for (i = 0; i < len; i++)
+		{
+			if (IS_HIGHBIT_SET(localebuf[i]))
+			{
+				skip = true;
+				break;
+			}
+		}
+		if (skip)
+		{
+			if (debug)
+				fprintf(stderr, _("%s: locale name has non-ASCII characters, skipped: %s\n"),
+						progname, localebuf);
+			continue;
+		}
+
+		enc = pg_get_encoding_from_locale(localebuf, debug);
+		if (enc < 0)
+		{
+			/* error message printed by pg_get_encoding_from_locale() */
+			continue;
+		}
+		if (!PG_VALID_BE_ENCODING(enc))
+			continue;			/* ignore locales for client-only encodings */
+		if (enc == PG_SQL_ASCII)
+			continue;			/* C/POSIX are already in the catalog */
+
+		count++;
+
+		quoted_locale = escape_quotes(localebuf);
+
+		PG_CMD_PRINTF3("INSERT INTO tmp_pg_collation VALUES (E'%s', E'%s', %d);\n",
+					   quoted_locale, quoted_locale, enc);
+
+		/*
+		 * Generate aliases such as "en_US" in addition to "en_US.utf8" for
+		 * ease of use.  Note that collation names are unique per encoding
+		 * only, so this doesn't clash with "en_US" for LATIN1, say.
+		 */
+		if (normalize_locale_name(alias, localebuf))
+			PG_CMD_PRINTF3("INSERT INTO tmp_pg_collation VALUES (E'%s', E'%s', %d);\n",
+						   escape_quotes(alias), quoted_locale, enc);
+	}
+
+	/* Add an SQL-standard name */
+	PG_CMD_PRINTF1("INSERT INTO tmp_pg_collation VALUES ('ucs_basic', 'C', %d);\n", PG_UTF8);
+
+	/*
+	 * When copying collations to the final location, eliminate aliases that
+	 * conflict with an existing locale name for the same encoding.  For
+	 * example, "br_FR.iso88591" is normalized to "br_FR", both for encoding
+	 * LATIN1.	But the unnormalized locale "br_FR" already exists for LATIN1.
+	 * Prefer the alias that matches the OS locale name, else the first locale
+	 * name by sort order (arbitrary choice to be deterministic).
+	 *
+	 * Also, eliminate any aliases that conflict with pg_collation's
+	 * hard-wired entries for "C" etc.
+	 */
+	PG_CMD_PUTS("INSERT INTO pg_collation (collname, collnamespace, collowner, collencoding, collcollate, collctype) "
+				" SELECT DISTINCT ON (collname, encoding)"
+				"   collname, "
+				"   (SELECT oid FROM pg_namespace WHERE nspname = 'pg_catalog') AS collnamespace, "
+				"   (SELECT relowner FROM pg_class WHERE relname = 'pg_collation') AS collowner, "
+				"   encoding, locale, locale "
+				"  FROM tmp_pg_collation"
+				"  WHERE NOT EXISTS (SELECT 1 FROM pg_collation WHERE collname = tmp_pg_collation.collname)"
+	   "  ORDER BY collname, encoding, (collname = locale) DESC, locale;\n");
+
+	pclose(locale_a_handle);
+	PG_CMD_CLOSE;
+
+	check_ok();
+	if (count == 0 && !debug)
+	{
+		printf(_("No usable system locales were found.\n"));
+		printf(_("Use the option \"--debug\" to see details.\n"));
+	}
+#else							/* not HAVE_LOCALE_T && not WIN32 */
+	printf(_("not supported on this platform\n"));
+	fflush(stdout);
+#endif   /* not HAVE_LOCALE_T  && not WIN32 */
+}
+
 /*
  * load conversion functions
  */
@@ -1870,7 +1928,7 @@ load_plpgsql(void)
 
 	PG_CMD_OPEN;
 
-	PG_CMD_PUTS("CREATE LANGUAGE plpgsql;\n");
+	PG_CMD_PUTS("CREATE EXTENSION plpgsql;\n");
 
 	PG_CMD_CLOSE;
 
@@ -1932,6 +1990,8 @@ make_template0(void)
 		"REVOKE CREATE,TEMPORARY ON DATABASE template1 FROM public;\n",
 		"REVOKE CREATE,TEMPORARY ON DATABASE template0 FROM public;\n",
 
+		"COMMENT ON DATABASE template0 IS 'unmodifiable empty database';\n",
+
 		/*
 		 * Finally vacuum to clean up dead rows in pg_database
 		 */
@@ -1967,6 +2027,7 @@ make_postgres(void)
 	const char **line;
 	static const char *postgres_setup[] = {
 		"CREATE DATABASE postgres;\n",
+		"COMMENT ON DATABASE postgres IS 'default administrative connection database';\n",
 		NULL
 	};
 
@@ -2176,7 +2237,7 @@ check_locale_encoding(const char *locale, int user_enc)
 {
 	int			locale_enc;
 
-	locale_enc = pg_get_encoding_from_locale(locale);
+	locale_enc = pg_get_encoding_from_locale(locale, true);
 
 	/* See notes in createdb() to understand these tests */
 	if (!(locale_enc == user_enc ||
@@ -2202,6 +2263,28 @@ check_locale_encoding(const char *locale, int user_enc)
 	return true;
 }
 
+#ifdef WIN32
+
+/*
+ * Replace 'needle' with 'replacement' in 'str' . Note that the replacement
+ * is done in-place, so 'replacement' must be shorter than 'needle'.
+ */
+static void
+strreplace(char *str, char *needle, char *replacement)
+{
+	char	   *s;
+
+	s = strstr(str, needle);
+	if (s != NULL)
+	{
+		int			replacementlen = strlen(replacement);
+		char	   *rest = s + strlen(needle);
+
+		memcpy(s, replacement, replacementlen);
+		memmove(s + replacementlen, rest, strlen(rest) + 1);
+	}
+}
+#endif   /* WIN32 */
 
 /*
  * set up the locale variables
@@ -2261,7 +2344,10 @@ setlocales(void)
 #ifdef WIN32
 typedef BOOL (WINAPI * __CreateRestrictedToken) (HANDLE, DWORD, DWORD, PSID_AND_ATTRIBUTES, DWORD, PLUID_AND_ATTRIBUTES, DWORD, PSID_AND_ATTRIBUTES, PHANDLE);
 
+/* Windows API define missing from some versions of MingW headers */
+#ifndef  DISABLE_MAX_PRIVILEGE
 #define DISABLE_MAX_PRIVILEGE	0x1
+#endif
 
 /*
  * Create a restricted token and execute the specified process with it.
@@ -2454,6 +2540,7 @@ main(int argc, char *argv[])
 		"pg_xlog/archive_status",
 		"pg_clog",
 		"pg_notify",
+		"pg_serial",
 		"pg_subtrans",
 		"pg_twophase",
 		"pg_multixact/members",
@@ -2589,8 +2676,8 @@ main(int argc, char *argv[])
 	}
 
 	if (strcmp(authmethod, "md5") &&
+		strcmp(authmethod, "peer") &&
 		strcmp(authmethod, "ident") &&
-		strncmp(authmethod, "ident ", 6) &&		/* ident with space = param */
 		strcmp(authmethod, "trust") &&
 #ifdef USE_PAM
 		strcmp(authmethod, "pam") &&
@@ -2619,6 +2706,20 @@ main(int argc, char *argv[])
 		exit(1);
 	}
 
+	/*
+	 * When ident is specified, use peer for local connections. Mirrored, when
+	 * peer is specified, use ident for TCP connections.
+	 */
+	if (strcmp(authmethod, "ident") == 0)
+		authmethodlocal = "peer";
+	else if (strcmp(authmethod, "peer") == 0)
+	{
+		authmethodlocal = "peer";
+		authmethod = "ident";
+	}
+	else
+		authmethodlocal = authmethod;
+
 	if (strlen(pg_data) == 0)
 	{
 		pgdenv = getenv("PGDATA");
@@ -2829,7 +2930,7 @@ main(int argc, char *argv[])
 	{
 		int			ctype_enc;
 
-		ctype_enc = pg_get_encoding_from_locale(lc_ctype);
+		ctype_enc = pg_get_encoding_from_locale(lc_ctype, true);
 
 		if (ctype_enc == -1)
 		{
@@ -2843,7 +2944,19 @@ main(int argc, char *argv[])
 		}
 		else if (!pg_valid_server_encoding_id(ctype_enc))
 		{
-			/* We recognized it, but it's not a legal server encoding */
+			/*
+			 * We recognized it, but it's not a legal server encoding. On
+			 * Windows, UTF-8 works with any locale, so we can fall back to
+			 * UTF-8.
+			 */
+#ifdef WIN32
+			printf(_("Encoding %s implied by locale is not allowed as a server-side encoding.\n"
+			   "The default database encoding will be set to %s instead.\n"),
+				   pg_encoding_to_char(ctype_enc),
+				   pg_encoding_to_char(PG_UTF8));
+			ctype_enc = PG_UTF8;
+			encodingid = encodingid_to_string(ctype_enc);
+#else
 			fprintf(stderr,
 					_("%s: locale %s requires unsupported encoding %s\n"),
 					progname, lc_ctype, pg_encoding_to_char(ctype_enc));
@@ -2852,6 +2965,7 @@ main(int argc, char *argv[])
 					"Rerun %s with a different locale selection.\n"),
 					pg_encoding_to_char(ctype_enc), progname);
 			exit(1);
+#endif
 		}
 		else
 		{
@@ -2899,7 +3013,7 @@ main(int argc, char *argv[])
 
 	printf("\n");
 
-	umask(077);
+	umask(S_IRWXG | S_IRWXO);
 
 	/*
 	 * now we are starting to do real work, trap signals so we can clean up
@@ -2924,7 +3038,7 @@ main(int argc, char *argv[])
 	pqsignal(SIGPIPE, SIG_IGN);
 #endif
 
-	switch (check_data_dir(pg_data))
+	switch (pg_check_dir(pg_data))
 	{
 		case 0:
 			/* PGDATA not there, must create it */
@@ -2946,7 +3060,7 @@ main(int argc, char *argv[])
 				   pg_data);
 			fflush(stdout);
 
-			if (chmod(pg_data, 0700) != 0)
+			if (chmod(pg_data, S_IRWXU) != 0)
 			{
 				fprintf(stderr, _("%s: could not change permissions of directory \"%s\": %s\n"),
 						progname, pg_data, strerror(errno));
@@ -2990,8 +3104,8 @@ main(int argc, char *argv[])
 			exit_nicely();
 		}
 
-		/* check if the specified xlog directory is empty */
-		switch (check_data_dir(xlog_dir))
+		/* check if the specified xlog directory exists/is empty */
+		switch (pg_check_dir(xlog_dir))
 		{
 			case 0:
 				/* xlog directory not there, must create it */
@@ -2999,7 +3113,7 @@ main(int argc, char *argv[])
 					   xlog_dir);
 				fflush(stdout);
 
-				if (mkdir_p(xlog_dir, 0700) != 0)
+				if (pg_mkdir_p(xlog_dir, S_IRWXU) != 0)
 				{
 					fprintf(stderr, _("%s: could not create directory \"%s\": %s\n"),
 							progname, xlog_dir, strerror(errno));
@@ -3010,13 +3124,14 @@ main(int argc, char *argv[])
 
 				made_new_xlogdir = true;
 				break;
+
 			case 1:
 				/* Present but empty, fix permissions and use it */
 				printf(_("fixing permissions on existing directory %s ... "),
 					   xlog_dir);
 				fflush(stdout);
 
-				if (chmod(xlog_dir, 0700) != 0)
+				if (chmod(xlog_dir, S_IRWXU) != 0)
 				{
 					fprintf(stderr, _("%s: could not change permissions of directory \"%s\": %s\n"),
 							progname, xlog_dir, strerror(errno));
@@ -3027,6 +3142,7 @@ main(int argc, char *argv[])
 
 				found_existing_xlogdir = true;
 				break;
+
 			case 2:
 				/* Present and not empty */
 				fprintf(stderr,
@@ -3104,6 +3220,8 @@ main(int argc, char *argv[])
 
 	setup_description();
 
+	setup_collation();
+
 	setup_conversion();
 
 	setup_dictionary();
